module Data.GraphRewriting.Morphism where

import Data.GraphRewriting.ExtendedEarl (Node)
import qualified Data.Map as Map (Map, lookup, empty, insert)
import qualified Data.Maybe as Maybe (fromMaybe)

-- | Morphism represents a Graph morphism between TermGraph A and
--  TermGraph B, and hence indicates a mapping from some nodes of A to 
--  some nodes of B
type Morphism = Map.Map Node Node 
           
-- | extend morphism (n,n'): extends morphism by the mapping n -> n' = m(n), 
--   Nothing, if (n,k) with k/=n' is in morphism already
extend :: Morphism -> (Node,Node) -> Maybe Morphism
extend  morphism (n,n') = case Map.lookup n  morphism of                         
  Nothing -> Just (Map.insert n n'  morphism)
  Just k  | k == n'   -> Just morphism
          | otherwise -> Nothing
                                                       
-- | mstar morphism n: if n is in the domain of morphism  then 
--   morphism(n) = n' else n 
mstar :: Morphism -> Node -> Node
mstar morphism n = Maybe.fromMaybe n (Map.lookup n morphism)

-- | empty: the empty morphism
empty :: Morphism
empty = Map.empty

-- | fromList ns: generates a morphism from the association list ns
--   associcating nodes with nodes.
fromList :: [(Node,Node)] -> Maybe Morphism
fromList ns = fromList' empty ns
  where fromList' morphism []            = Just morphism
        fromList' morphism ((n,n'):ns')  = case extend morphism (n,n') of 
          Just morph' -> fromList' morph' ns' 
          Nothing     -> Nothing